/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * simple callback application
 * 
 * Copyright (C) 2004, Andy Powell 
 *
 */

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/channel_pvt.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/config.h>
#include <asterisk/say.h>
#include <asterisk/module.h>
#include <asterisk/app.h>
#include <asterisk/manager.h>
#include <asterisk/localtime.h>
#include <asterisk/cli.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <dirent.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <ctype.h>
#include <sys/file.h>
#include "../astconf.h"

#define DEFAULT_MAXRETRIES 3
#define DEFAULT_RETRYTIME 60
#define DEFAULT_WAITTIME 60
						

static char *tdesc = "Generate call file";

static char *app_callback = "Callback";

static char *callback_synopsis = "Callback([context,]exten,priority,dialout-context[,waittime][,retrytime][,retries])";

static char *callback_descrip =
"callback([context,]exten,priority,dialout-context[,waittime][,retrytime][,retries])\n\n"
"Generate a call file to call a user back, puts them into context, at exten priority\n"
"via the dialout-context. You may also specify the standard spool file values for\n"
"waittime, retrytime and retries. The accountcode is taken from the current channel\n"
"\n";


STANDARD_LOCAL_USER;

LOCAL_USER_DECL;


static int callback_exec(struct ast_channel *chan, void *data)
{
	int res=0;
        int inum;
        FILE *p=NULL;
        int pfd;
		
	char *s;
        char *ccontext,*cexten, *cpriority, *cdialout;
	char *cwaittime, *cretrytime, *cretries;

	int iwaittime, iretrytime, iretries;

	struct localuser *u;

  	char old_cid[144] = "", *num, *name;
  
        char tmp[80] = "/tmp/callback-XXXXXX";
        char tmp2[256];
        time_t t;
        struct tm tm;
	
	if (!data) {
		ast_log(LOG_WARNING, "You didn't pass any arguments - I need at least exten, priority and dialout context\n");
		return -1;
	}

	LOCAL_USER_ADD(u);
	/* Every time you say you'tr learning, you... */

        s = ast_strdupa((void *) data);

        ccontext      = strsep(&s, "|");
        cexten        = strsep(&s, "|");
        cpriority     = strsep(&s, "|");
        cdialout      = strsep(&s, "|");
	cwaittime     = strsep(&s, "|");
        cretrytime    = strsep(&s, "|");
        cretries      = strsep(&s, "|");

	if (ccontext && cexten && cpriority && !cdialout) {
		cretries  = cretrytime;
		cretrytime = cwaittime;
		cwaittime = cdialout; 
		cdialout = cpriority;
		cpriority = cexten;
		cexten = ccontext;
		ccontext = chan->context;	
	}

        /* Just in case they really try to push their luck check as
           much as possible for user stupidity */

		   

        if (!cwaittime || !strcmp(cwaittime,"")) {
                iwaittime = DEFAULT_WAITTIME;
        }
	else {
	        if (sscanf(cwaittime, "%d", &iwaittime) != 1) {
			ast_log(LOG_WARNING, "'%s' is not a valid number\n", cwaittime);
			LOCAL_USER_REMOVE(u);
			return -1;
	        }
	}


        if (!cretrytime || !strcmp(cretrytime,"")) {
                iretrytime = DEFAULT_RETRYTIME;
        }
	else {
	        if (sscanf(cretrytime, "%d", &iretrytime) != 1) {
	                ast_log(LOG_WARNING, "'%s' is not a valid number\n", cretrytime);
	                LOCAL_USER_REMOVE(u);
	                return -1;
	        }
								
	}


        if (!cretries || !strcmp(cretries,"")) {
		iretries = DEFAULT_MAXRETRIES;	
        }
	else {
		if (sscanf(cretries, "%d", &iretries) != 1) {
	        	ast_log(LOG_WARNING, "'%s' is not a valid number\n", cretries);
	        	LOCAL_USER_REMOVE(u);
			return -1;
							        }
								
	}

	if (!strcmp(ccontext,""))
		ccontext = chan->context;
		
	/* check to see if params passed */

        if (!strcmp(ccontext,"")) {
                ast_log(LOG_WARNING, "Callback needs at least an exten, priority and a dialout context\n");
                LOCAL_USER_REMOVE(u);
                return -1;
	}						


        if (!cpriority || !cexten || !cdialout) {
                ast_log(LOG_WARNING, "Callback needs at least an exten, priority and a dialout context\n");
                LOCAL_USER_REMOVE(u);
                return -1;
	}	


	strncpy (old_cid, chan->callerid, sizeof (old_cid) - 1);
        ast_callerid_parse (old_cid, &name, &num);        /* this destroys the original string */
	    
	if (num)       
		ast_verbose( VERBOSE_PREFIX_3 "Generating call to %s %s,%s,%s from %s\nRetries:%d RetryTime:%d WaitTime:%d\n",
				num ,ccontext,cexten,cpriority,cdialout,iretries,iretrytime,iwaittime);
	else {
		ast_verbose( VERBOSE_PREFIX_3 "Caller id is useless to us, aborting\n");
                LOCAL_USER_REMOVE(u);
                return -1;
	}


        pfd = mkstemp(tmp);

        if (pfd > -1) {
                p = fdopen(pfd, "w");
                if (!p) {
                        close(pfd);
                        pfd = -1;
                }
        }

        if (p) {
		fprintf(p,"Channel: LOCAL/%s@%s\n",num,cdialout);
		fprintf(p,"MaxRetries: %d\n",iretries);
		fprintf(p,"RetryTime: %d\n",iretrytime);
		fprintf(p,"WaitTime: %d\n", iwaittime);
		fprintf(p,"Context: %s\n",ccontext);
		fprintf(p,"Extension: %s\n",cexten);
		fprintf(p,"Priority: %s\n",cpriority);
                fprintf(p,"CallerID: Callback <%s>\n",chan->exten);
                fprintf(p,"Account: %s\n",chan->accountcode);

                fclose(p);

		res = ast_safe_sleep(chan, 10000);

	        if (chan->_state == AST_STATE_UP)
                        res = ast_hangup(chan);
					
                snprintf(tmp2, sizeof(tmp2), "( mv %s %s/outgoing/ ) &", tmp,ast_config_AST_SPOOL_DIR);
                ast_safe_system(tmp2);
	}


	LOCAL_USER_REMOVE(u);
	return res;
}



int unload_module(void)
{
     int res;
     STANDARD_HANGUP_LOCALUSERS;
     res = ast_unregister_application(app_callback);
     return res;
}

int load_module(void)
{
     int res;
     res = ast_register_application(app_callback, callback_exec, callback_synopsis, callback_descrip);

     return res;
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}



/* Fading everything to black and blue... */




